// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/httptasker"
	"github.com/jaryhe/gopkgs/util"
	"go.uber.org/zap"
	"net/http"
	"smart-government-management-gateway/consts"
	"smart-government-management-gateway/errors"
	param_v1 "smart-government-management-gateway/param/v1"
	"smart-government-management-gateway/pb"
	"smart-government-management-gateway/pb/v1"
//	"smart-government-management-gateway/utils"
	"strconv"
	"strings"
)

// 设备类型总列表
// @Summary 设备类型总列表
// @Description 设备类型总列表
// @Tags device
// @Accept  json
// @Produce  json
// @Success 200 {object} v1.DeviceAllTypeListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device_type_all [get]
func (c *Controller) DeviceTypeAll(ctx *gin.Context) {

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceAllTypeListResponse{}
		rpcReq := &v1.DeviceAllTypeListRequest{}
		reply, err := pb.Government.DeviceAllTypeList(ctx, rpcReq)
		if err != nil {

			logger.Error("func",
				zap.String("call", "DeviceAllTypeList"),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List= make([]*v1.DeviceAllTypeItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, handleServiceTask)
}

// 已通过审核的设备类型列表
// @Summary 已通过审核的设备类型列表
// @Description 已通过审核的设备类型列表
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param filter query string false " "
// @Param page query int true " "
// @Param provider_id query int false " "
// @Param device_code query int false " "
// @Param list_type query int true "0 已对接的设备列表 1 待审批的设备申请列表 2 待审批的设备对接列表"
// @Success 200 {object} v1.DeviceTypeListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/type_list [get]
func (c *Controller) DeviceTypeList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceTypeListRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.DeviceTypeListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceTypeListResponse{}
		rpcReq := &v1.DeviceTypeListRequest{}
		rpcReq.Page = req.Page
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}
		rpcReq.Filter = req.Filter
		rpcReq.ListType = uint32(req.ListType)
		rpcReq.ProviderId = req.ProviderId
		rpcReq.DeviceCode = req.DeviceCode
		reply, err := pb.Government.DeviceTypeList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceTypeList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.DeviceTypeItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)

}


//
// @Summary 扬尘设备列表
// @Description 扬尘设备列表
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param filter query string false " "
// @Param project_id query int false " "
// @Param provider_id query int false " "
// @Param cid query int false " "
// @Param state query int true " "
// @Success 200 {object} v1.DeviceListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/dust_list [get]
func (c *Controller) DeviceDustList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceListRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.DeviceListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceListResponse{}
		rpcReq := &v1.DeviceListRequest{}
		rpcReq.Page = req.Page
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}
		rpcReq.Filter = req.Filter
		req.FilterStatus = "1"
		rpcReq.TypeCode = consts.DeviceTypeDust
		rpcReq.State = req.State
		rpcReq.ProjectId = req.ProjectId
		rpcReq.Cid = req.Cid
		rpcReq.ProviderId = req.ProviderId
		if req.FilterStatus != "" {
			array := strings.Split(req.FilterStatus, ",")
			for _, v := range array {
				value, _ := strconv.Atoi(v)
				rpcReq.StatusFilters = append(rpcReq.StatusFilters, int32(value))
			}
		}

		reply, err := pb.Government.DeviceList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.DeviceItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 实名制设备列表
// @Description 实名制设备列表
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param filter query string false " "
// @Param project_id query int false " "
// @Param provider_id query int false " "
// @Param cid query int false " "
// @Param state query int true " "
// @Success 200 {object} v1.DeviceListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/attendance_list [get]
func (c *Controller) DeviceAttendanceList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceListRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.DeviceListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceListResponse{}
		rpcReq := &v1.DeviceListRequest{}
		rpcReq.Page = req.Page
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}
		rpcReq.Filter = req.Filter
		req.FilterStatus = "1"
		rpcReq.TypeCode = consts.DeviceTypeAttendance
		rpcReq.State = req.State
		rpcReq.ProjectId = req.ProjectId
		rpcReq.Cid = req.Cid
		rpcReq.ProviderId = req.ProviderId
		if req.FilterStatus != "" {
			array := strings.Split(req.FilterStatus, ",")
			for _, v := range array {
				value, _ := strconv.Atoi(v)
				rpcReq.StatusFilters = append(rpcReq.StatusFilters, int32(value))
			}
		}

		reply, err := pb.Government.DeviceList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.DeviceItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 视频设备列表
// @Summary 视频设备列表
// @Description 视频设备列表
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param filter query string false "过滤字段"
// @Param project_id query int false " "
// @Param provider_id query int false " "
// @Param cid query int false " "
// @Param state query int true " "
// @Param page query int false " "
// @Success 200 {object} v1.VedioListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/vedio_list [get]
func (c *Controller) DeviceVedioList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.VedioListRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.VedioListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, _, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.VedioListResponse{}
		rpcReq := &v1.VedioListRequest{}
		rpcReq.Page = req.Page
		rpcReq.ProviderId = req.ProviderId
		rpcReq.Filter = req.Filter
		rpcReq.State = req.State
		rpcReq.ProjectId = req.ProjectId
		rpcReq.Cid = req.Cid
		rpcReq.ProviderId = req.ProviderId
		/*
			if req.FilterStatus != "" {
				array := strings.Split(req.FilterStatus, ",")
				for _, v := range array {
					value, _ := strconv.Atoi(v)
					rpcReq.StatusFilters = append(rpcReq.StatusFilters, int32(value))
				}
			}*/
		rpcReq.StatusFilters = append(rpcReq.StatusFilters, 1)
		reply, err := pb.Government.VedioList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "VedioList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.VedioItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 设备拆机工单列表
// @Summary 设备拆机工单列表
// @Description 设备拆机工单列表
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param filter query string false " "
// @Param filter_status query string false "状态，逗号分隔:0 待审核 1 通过 2 未通过"
// @Success 200 {object} v1.DeviceDelJobListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/del_job_list [get]
func (c *Controller) DeviceDelJobList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceDelJobListRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.DeviceDelJobListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		//loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.DeviceDelJobListResponse{}
		rpcReq := &v1.DeviceDelJobListRequest{
			Page:req.Page,
			Filter:req.Filter,
		}
		req.FilterStatus = fmt.Sprintf("%d,%d", consts.StatusAuditting, consts.StatusAuditedFail)
		if req.FilterStatus != "" {
			array := strings.Split(req.FilterStatus, ",")
			for _, v := range array {
				value, _ := strconv.Atoi(v)
				rpcReq.StatusFilters = append(rpcReq.StatusFilters, int32(value))
			}
		}
		reply, err := pb.Government.DeviceDelJobList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceDelJobList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.DeviceDelJobItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 通道列表
// @Summary 通道列表
// @Description 通道列表
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param sn query string true " "
// @Param page query int false " "
// @Param channel_no query string false " "
// @Success 200 {object} v1.ChannelListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/vedio_channel_list [get]
func (c *Controller) DeviceVedioChannelList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ChannelListRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ChannelListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.ChannelListResponse{}
		rpcReq := &v1.ChannelListRequest{}
		rpcReq.Page = req.Page
		rpcReq.Sn = req.Sn
		rpcReq.ChannelNo = req.ChannelNo
		/*
			if req.FilterStatus != "" {
				array := strings.Split(req.FilterStatus, ",")
				for _, v := range array {
					value, _ := strconv.Atoi(v)
					rpcReq.StatusFilters = append(rpcReq.StatusFilters, int32(value))
				}
			}*/
		reply, err := pb.Government.ChannelList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ChannelList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.ChannelItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


//
// @Summary 设备统计
// @Description 设备统计
// @Tags 前台
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.DeviceStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/statistic [get]
func (c *Controller) DeviceStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceStatisticRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceStatisticResponse{}
		rpcReq := &v1.DeviceStatisticRequest{}

		reply, err := pb.Government.DeviceStatistic(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 设备类型统计
// @Description 设备类型统计
// @Tags 前台
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.DeviceTypeStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/type_statistic [get]
func (c *Controller) DeviceTypeStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceTypeStatisticRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceTypeStatisticResponse{}
		rpcReq := &v1.DeviceTypeStatisticRequest{}

		reply, err := pb.Government.DeviceTypeStatistic(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceTypeStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if reply.List == nil {
			reply.List = []*v1.DeviceTypeStatisticItem{}
		}
		resp.Data = *reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


//
// @Summary 已通过审核的设备详细申请信息
// @Description 已通过审核的设备详细申请信息
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id path int true " "
// @Success 200 {object} v1.DeviceTestedTypeInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/tested_type_info/{id} [get]
func (c *Controller) DeviceTestedTypeInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.DeviceTestedTypeInfoRequest{}
	//var loginUid int64
	//var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, &req.DeviceTestedTypeInfoPath, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
	//	loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.DeviceTestedTypeInfoResponse{}
		rpcReq := &v1.DeviceTestedTypeInfoRequest{}
		rpcReq.Id = req.Id
		reply, err := pb.Government.DeviceTestedTypeInfo(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "DeviceTestedTypeInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)

}
